@@ -15,12 +15,17 @@ Defines all the base classes for response objects.
 """
 
 from datetime import datetime, timedelta
+import json
 import logging
+import six
 import time
 
-from ambariclient import events, exceptions
+from ambariclient import events, exceptions, utils
 
 LOG = logging.getLogger(__name__)
+LOG.addHandler(utils.NullHandler())
+
+OLDEST_SUPPORTED_VERSION = (1, 7, 0)
 
 
 class PollableMixin(object):
@@ -122,15 +127,20 @@ class ModelCollection(object):
 
     def __iter__(self):
         self.inflate()
+        self._iter_marker = 0
         return self
 
     def next(self):
+        self.inflate()
         if self._iter_marker >= len(self._models):
             raise StopIteration
         model = self._models[self._iter_marker]
         self._iter_marker += 1
         return model
 
+    def __next__(self):
+        return self.next()
+
     def __call__(self, *args):
         raise NotImplementedError("'__call__' must be defined by subclasses")
 
@@ -146,7 +156,7 @@ class ModelCollection(object):
         return
 
     @events.evented
-    def wait(self, **kwargs):
+    def wait(self, **kwargs):  # pylint: disable=unused-argument
         """Wait until the collection is loaded."""
         return self.inflate()
 
@@ -173,10 +183,11 @@ class QueryableModelCollection(ModelCollection):
     when you act on the host objects it contains.
     """
     def __init__(self, *args, **kwargs):
-        self.request = None
         super(QueryableModelCollection, self).__init__(*args, **kwargs)
+        self.request = None
+        self._filter = {}
 
-    def __call__(self, *args):
+    def __call__(self, *args, **kwargs):
         if len(args) == 1:
             if isinstance(args[0], list):
                 # allow for passing in a list of ids and filtering the set
@@ -201,8 +212,19 @@ class QueryableModelCollection(ModelCollection):
                     model = self.model_class(self, href='/'.join([self.url, item]),
                                              data={self.model_class.primary_key: item})
                 self._models.append(model)
+            return self
 
-        # TODO - pagination support?  Other filters on collections?
+        self._is_inflated = False
+        self._filter = {}
+        self._models = []
+        if kwargs:
+            prefix = self.model_class.data_key
+            for (key, value) in kwargs.iteritems():
+                if self.model_class.use_key_prefix:
+                    key = '/'.join([prefix, key])
+                if not isinstance(value, six.string_types):
+                    value = json.dumps(value)
+                self._filter[key] = value
 
         return self
 
@@ -222,7 +244,8 @@ class QueryableModelCollection(ModelCollection):
     def inflate(self):
         """Load the collection from the server, if necessary."""
         if not self._is_inflated:
-            self.load(self.client.get(self.url))
+            self.check_version()
+            self.load(self.client.get(self.url, params=self._filter))
 
         self._is_inflated = True
         return self
@@ -291,6 +314,15 @@ class QueryableModelCollection(ModelCollection):
             self.request = None
         return self.inflate()
 
+    def check_version(self):
+        if (self.model_class.min_version > OLDEST_SUPPORTED_VERSION
+                and self.client.version < self.model_class.min_version):
+            min_version = utils.version_str(self.model_class.min_version)
+            curr_version = utils.version_str(self.client.version)
+            raise exceptions.ClientError(message="Cannot access %s in version %s, it was added in "
+                                                 "version %s" % (self.url, curr_version,
+                                                                 min_version))
+
 
 class DependentModelCollection(ModelCollection):
     """A collection of DependentModel objects.
@@ -316,8 +348,18 @@ class DependentModelCollection(ModelCollection):
         reloaded, it should create new collections for these resources.
         """
         items = []
-        if len(args) == 1 and isinstance(args[0], list):
-            items = args[0]
+        if len(args) == 1:
+            if isinstance(args[0], list):
+                items = args[0]
+            else:
+                matches = [x for x in self._models if x.identifier == args[0]]
+                if len(matches) == 1:
+                    return matches[0]
+                elif len(matches) > 1:
+                    raise ValueError("More than one {0} with {1} '{2}' found in "
+                                     "collection".format(self.model_class.__class__.__name__,
+                                                         self.model_class.primary_key, args[0]))
+                return None
 
         if len(items) > 0:
             self._models = []
@@ -356,14 +398,22 @@ class Model(object):
     primary_key = None
     fields = []
     relationships = {}
+    min_version = OLDEST_SUPPORTED_VERSION
 
     def __init__(self, parent, data=None):
-        self._data = data if data is not None else {}
+        if data is None:
+            data = {}
+
+        self._data = dict((key, value) for key, value in six.iteritems(data)
+                          if key in set(self.fields))
         self.parent = parent
         self.client = parent.client
         self._is_inflated = False
         self._relationship_cache = {}
 
+    def __dir__(self):
+        return self.__dict__.keys() + list(self.fields) + self.relationships.keys()
+
     @property
     def identifier(self):
         """A model's identifier is the value of its primary key."""
@@ -383,9 +433,12 @@ class Model(object):
         they won't do an http request unless they have to.
         """
         if attr in self.relationships:
-            self.inflate()
+            rel_class = self.relationships[attr]
+            # we can't lazy-load DependentModel types
+            if issubclass(rel_class, DependentModel):
+                self.inflate()
+
             if attr not in self._relationship_cache:
-                rel_class = self.relationships[attr]
                 self._relationship_cache[attr] = rel_class.collection_class(
                     self.client, rel_class,
                     parent=self,
@@ -454,7 +507,7 @@ class Model(object):
         return { self.primary_key: self.identifier }
 
     @events.evented
-    def wait(self, **kwargs):
+    def wait(self, **kwargs):  # pylint: disable=unused-argument
         """Calling wait() on a model makes it wait until the object is in a valid state.
 
         So, for example, if you wait() on a cluster after creating it, it will
@@ -515,6 +568,7 @@ class QueryableModel(Model):
     """
 
     collection_class = QueryableModelCollection
+    use_key_prefix = True
     path = None
     data_key = None
     relationships = {}
@@ -549,10 +603,8 @@ class QueryableModel(Model):
                 # catch infinite recursion when attempting to inflate
                 # an object that doesn't have enough data to inflate
                 msg = ("There is not enough data to inflate this object.  "
-                       "Need either an href: {} or a {}: {}").format(
-                          self._href, self.primary_key,
-                          self._data.get(self.primary_key)
-                      )
+                       "Need either an href: {} or a {}: {}")
+                msg = msg.format(self._href, self.primary_key, self._data.get(self.primary_key))
                 raise exceptions.ClientError(msg)
 
             self._is_inflating = True
@@ -598,7 +650,7 @@ class QueryableModel(Model):
             if self.data_key and self.data_key in response:
                 self._data.update(response.pop(self.data_key))
                 # preload related object collections, if received
-                for rel in [x for x in self.relationships if x in response]:
+                for rel in [x for x in self.relationships if x in response and response[x]]:
                     rel_class = self.relationships[rel]
                     collection = rel_class.collection_class(
                         self.client, rel_class, parent=self
