@@ -20,10 +20,12 @@ from ambariclient import models, utils, base, exceptions
 from ambariclient.exceptions import handle_response
 
 LOG = logging.getLogger(__name__)
+LOG.addHandler(utils.NullHandler())
 
 # this defines where the Ambari client delegates to for actual logic
 ENTRY_POINTS = {
     'actions': models.Action,
+    'alert_targets': models.AlertTarget,
     'blueprints': models.Blueprint,
     'bootstrap': models.Bootstrap,
     'clusters': models.Cluster,
@@ -35,10 +37,7 @@ ENTRY_POINTS = {
     'views': models.View,
 }
 
-OLDEST_SUPPORTED_VERSION = (1, 7, 0)
 
-# TODO: flesh out version handling, this is weaksauce
-# TODO: Sphinx docs
 class Ambari(object):
     """The Ambari client
 
@@ -58,7 +57,8 @@ class Ambari(object):
 
     """
     def __init__(self, host, port=None, username=None, password=None,
-                 identifier=None, protocol=None, validate_ssl=True, max_retries=5):
+                 identifier=None, protocol=None, validate_ssl=True,
+                 timeout=10, max_retries=5):
 
         self.base_url = utils.generate_base_url(host, port=port, protocol=protocol)
 
@@ -67,16 +67,19 @@ class Ambari(object):
 
         self.client = HttpClient(host=self.base_url, username=username,
                                  password=password, identifier=identifier,
-                                 validate_ssl=validate_ssl, max_retries=max_retries)
+                                 validate_ssl=validate_ssl, timeout=timeout,
+                                 max_retries=max_retries)
         self._version = None
 
-    # TODO: make this check automatic at some point
+    def __dir__(self):
+        return self.__dict__.keys() + ENTRY_POINTS.keys()
+
     def check_version(self):
-        if self.version < OLDEST_SUPPORTED_VERSION:
+        if self.version < base.OLDEST_SUPPORTED_VERSION:
             raise exceptions.ClientError(
                 "Version %s unsupported, must be %s or higher"
                 % (utils.version_str(self.version),
-                   utils.version_str(OLDEST_SUPPORTED_VERSION)))
+                   utils.version_str(base.OLDEST_SUPPORTED_VERSION)))
         return
 
     @property
@@ -112,12 +115,13 @@ class HttpClient(object):
     cases do exist either due to Ambari bugs or other mitigating circumstances.
     """
     def __init__(self, host, username, password, identifier, validate_ssl=True,
-                 max_retries=5):
+                 timeout=10, max_retries=5):
 
         self.request_params = {
             'headers': {'X-Requested-By': identifier},
             'auth': (username, password),
             'verify': validate_ssl,
+            'timeout': timeout,
         }
         # automatically retry requests on connection errors
         self.session = requests.Session()
@@ -132,11 +136,11 @@ class HttpClient(object):
 
         if content_type is not None:
             params['headers']['Content-type'] = content_type
-        LOG.debug("Request headers: %s" % params['headers'])
+        LOG.debug("Request headers: %s", params['headers'])
 
         if 'data' in params and isinstance(params['data'], dict):
             params['data'] = json.dumps(params['data'], cls=AmbariJsonEncoder)
-            LOG.debug("Request body: %s" % params['data'])
+            LOG.debug("Request body: %s", params['data'])
 
         response = requests_method(url, **params)
 
@@ -148,8 +152,8 @@ class HttpClient(object):
 
         if response.headers.get('content-length') is None:
             # Log bad methods so we can report them
-            LOG.info("Missing content-length for %s %s: %s", method,
-                     url, response.headers.get('content-type'))
+            LOG.debug("Missing content-length for %s %s: %s", method,
+                      url, response.headers.get('content-type'))
 
         # there is no consistent way to determine response type
         # so assume json if it's not an empty string
@@ -174,7 +178,7 @@ class AmbariJsonEncoder(json.JSONEncoder):
     This allows for passing in models and ModelCollections into related objects'
     create/update methods and having it handle the conversion automatically.
     """
-    def default(self, obj):
+    def default(self, obj):  # pylint: disable=method-hidden
         if isinstance(obj, base.ModelCollection):
             dicts = []
             for model in obj:
@@ -183,4 +187,4 @@ class AmbariJsonEncoder(json.JSONEncoder):
         elif isinstance(obj, base.Model):
             return obj.to_json_dict()
         # Let the base class default method raise the TypeError
-        return json.JSONEncoder.default(self, obj)
+        return super(AmbariJsonEncoder, self).default(obj)
